/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.3 $ 
 *     $Date: 2003/10/22 13:04:53 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "ATable.h"
#include "HiRes.h"

ATable::ATable() {
    scrollBar=NULL;
    pos.topLeft.x=0;
    pos.topLeft.y=0;
    pos.extent.x=0;
    pos.extent.y=0;
    rowHeight=1;
    scrollBar=NULL;
    upperRow=0;
    currentRow=-1;
}

void ATable::init(FormPtr fm,UInt16 tID,UInt16 scrollbarID) {
    upperRow=0;
    form=fm;
    tableID=tID;

    scrollBar=(ScrollBarPtr)FrmGetObjectPtr(form,FrmGetObjectIndex(form,scrollbarID));
    resize();

    // calculate row height and column widths
    rowHeight=AFntCharHeight();
    columnWidths.clear();
}

void ATable::resize() {
    AFrmGetObjectBounds(form,FrmGetObjectIndex(form,tableID),&pos);
    AWinEraseRectangle(&pos,0);
}

void ATable::addColumn(Coord width) {
    columnWidths.add(width);
}

void ATable::redraw(Boolean needSetScroll) {
    Int16 row,col,mrow;
    RectangleType rect;

    checkUpperRow();

    rect.topLeft.y=pos.topLeft.y;
    rect.extent.y=rowHeight;
    for (row=0; row<getNumberOfRowsVisible(); row++) {
        if (row+upperRow >= getRowCount()) break;
        rect.topLeft.x=pos.topLeft.x;
        for (col=0; col<(Int16)columnWidths.size(); col++) {
            rect.extent.x=columnWidths.get(col);
            drawCell(row+upperRow,col,&rect);
            rect.topLeft.x+=rect.extent.x;
        }
        rect.topLeft.y+=rect.extent.y;
    }
    rect.topLeft.x=pos.topLeft.x; rect.extent.x=pos.extent.x;
    rect.extent.y=pos.topLeft.y+pos.extent.y-rect.topLeft.y;
    AWinEraseRectangle(&rect,0);
    if (needSetScroll)
	setScrollBarPos();
    invertCurrentRow();
}

Boolean ATable::getCellRect(Int16 row, Int16 col, RectangleType *rect) {
    Int16 mrow,orow;

    orow=row-upperRow;
    if (getNumberOfRowsVisible()<getRowCount())
        mrow=getNumberOfRowsVisible();
    else
        mrow=getRowCount();
    if (orow<0 || orow>mrow) return false;
    if (col>=(Int16)columnWidths.size()) return false;

    rect->topLeft.y=pos.topLeft.y+rowHeight*orow;
    rect->extent.y=rowHeight;
    rect->topLeft.x=pos.topLeft.x;
    for (Int16 icol=0; icol<col; icol++)
        rect->topLeft.x+=columnWidths.get(icol);
    rect->extent.x=columnWidths.get(col);
    return true;
}

void ATable::redrawCell(Int16 row,Int16 col) {
    RectangleType rect;

    if (getCellRect(row, col, &rect)) {
	drawCell(row,col,&rect);
    }
}

inline Int16 ATable::getNumberOfRowsVisible() {
    return (pos.extent.y)/(rowHeight);
}

void ATable::setScrollBarPos() {
    Int16 ScrMax;
    if (getRowCount()>=getNumberOfRowsVisible())
        ScrMax=getRowCount()-getNumberOfRowsVisible();
    else
        ScrMax=0;
    SclSetScrollBar(scrollBar,upperRow,0,ScrMax,getNumberOfRowsVisible()-1);
}


void ATable::setScroll(Int16 newValue) {
    upperRow=newValue;
    checkUpperRow();
    redraw(false);
}

void ATable::checkUpperRow() {
    if (upperRow>getRowCount()-getNumberOfRowsVisible()) upperRow=getRowCount()-getNumberOfRowsVisible();
    if (upperRow<0) upperRow=0;
}

void ATable::doPageUp() {
    Int16 oldup=upperRow;
    upperRow-=getNumberOfRowsVisible()-1;
    checkUpperRow();
    if (upperRow!=oldup)
        redraw(true);
}

void ATable::doPageDown() {
    Int16 oldup=upperRow;
    upperRow+=getNumberOfRowsVisible()-1;
    checkUpperRow();
    if (upperRow!=oldup)
        redraw(true);
}

Boolean ATable::getRowColByPoint(Coord x,Coord y,Int16 *row,Int16 *col) {
    x-=pos.topLeft.x;
    y-=pos.topLeft.y;

    if (x<0 || x>=(pos.extent.x)) return false;
    if (y<0 || y>=(pos.extent.y)) return false;
    if (y/rowHeight>=getNumberOfRowsVisible()) return false;

    *row=y/rowHeight+upperRow;
    if (*row >= getRowCount()) return false;

    UInt16 cw;
    for (UInt16 i=0; i<columnWidths.size(); i++) {
        cw=columnWidths.get(i);
        if (x<(Int16)cw) {
            *col=i;
            return true;
        } else
            x-=cw;
    }
    return false;
}

Boolean ATable::event(EventPtr e) {
    Int16 row=0,col=0;

    switch (e->eType) {
    case penDownEvent:
	if (getRowColByPoint((Coord)(ACoordStdToReal(e->screenX)),(Coord)(ACoordStdToReal(e->screenY)),&row,&col)) {
	    selectCell(row,col);
            return true;
	} else {
            return false;
	}
    case keyDownEvent:
        switch (e->data.keyDown.chr) {
        case vchrPageUp:
        case 0x1700:
            doPageUp();
            return true;
        case vchrPageDown:
        case 0x1701:
            doPageDown();
            return true;
        }
	break;
    case sclExitEvent:
        SclHandleEvent(scrollBar,e);
	setScroll(e->data.sclExit.newValue);
        return true;
    case sclRepeatEvent:
        SclHandleEvent(scrollBar,e);
	setScroll(e->data.sclRepeat.newValue);
	e->data.sclRepeat.time=100;
        return true;
    default: break;
    }
    return false;
}

void ATable::invertCurrentRow() {
    RectangleType rect;

    if (currentRow>=0) {
	if (getCellRect(currentRow,0,&rect)) {
	    rect.extent.x = pos.extent.x;
	    AWinInvertRectangle(&rect,0);
	}
    }
}

void ATable::setCurrentRow(Int16 row) {
    invertCurrentRow();
    currentRow = row;
    invertCurrentRow();
}

void ATable::drawCell(Int16 row,Int16 col,RectangleType *rect) {}
Int16 ATable::getRowCount() {return 0;}
void ATable::selectCell(Int16 row,Int16 col) {}
