/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "Utils.h"

Err MemStreamWriter::writeToDB(DmOpenRef db,UInt16 *index) {
    MemHandle mem;
    UInt8 *data;
    Err e;

    if ((mem=DmNewRecord(db,index,ssize))==NULL) return DmGetLastErr();
    data=(UInt8 *)MemHandleLock(mem);
    e=DmWrite(data,0,stream,ssize);
    if (e!=errNone) return e;
    MemHandleUnlock(mem);
    e=DmReleaseRecord(db,*index,true);
    if (e!=errNone) return e;
    return errNone;
}

void MemStreamWriter::writeUInt8(UInt8 value) {
    if ((allocated-ssize) < 1) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+1, 1))
	    return;
    }
    *(UInt8 *)(stream+ssize) = value;
    ssize+=sizeof(UInt8);
}
void MemStreamWriter::writeUInt8aligned(UInt8 value) {
    if ((allocated-ssize) < 2) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+2, 1))
	    return;
    }
    *(UInt8 *)(stream+ssize) = value;
    ssize+=2;
}
void MemStreamWriter::writeUInt16(UInt16 value) {
    if ((allocated-ssize) < sizeof(UInt16)) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+sizeof(UInt16), 1))
	    return;
    }
    *(UInt16 *)(stream+ssize) = value;
    ssize+=sizeof(UInt16);
}
void MemStreamWriter::writeUInt32(UInt32 value) {
    if ((allocated-ssize) < sizeof(UInt32)) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+sizeof(UInt32), 1))
	    return;
    }
    *(UInt32 *)(stream+ssize) = value;
    ssize+=sizeof(UInt32);
}
void MemStreamWriter::writeInt32(Int32 value) {
    if ((allocated-ssize) < sizeof(Int32)) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+sizeof(Int32), 1))
	    return;
    }
    *(Int32 *)(stream+ssize) = value;
    ssize+=sizeof(Int32);
}
void MemStreamWriter::writeBlock(MemPtr in, UInt16 len) {
    if ((allocated-ssize) < len) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+len, 1))
	    return;
    }
    MemMove(stream+ssize,in,len);
    ssize+=len;
}

void MemStreamWriter::writeString(char *s) {
    // write length and NUL terminated string
    UInt16 len=StrLen(s);
    if ((allocated-ssize) < len+sizeof(UInt16)+2) {
	if (!UtilsTemplatesRealloc(&(void *)stream, allocated, ssize+len+sizeof(UInt16)+2, 1))
	    return;
    }
    *(UInt16 *)(stream+ssize)=len;
    ssize+=sizeof(UInt16);
    MemMove(stream+ssize,s,len);
    *(char *)(stream+ssize+len)='\0';
    len++;
    if (len%2==1)
        len++;
    ssize+=len;
}
