/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#ifndef apUtils
#define apUtils
#include <PalmOS.h>
#include "UtilsTemplates.h"

inline Int16 abs(Int16 x) {
    return x>0 ? x : -x;
}

class MemStreamReader {
protected:
    UInt8 *stream, *pos;
public:
    MemStreamReader(MemPtr ptr) {
	stream = (UInt8 *)ptr;
        pos = (UInt8 *)ptr;
    }
    inline UInt8 readBoolean() {
	return readUInt8()!=0;
    }
    inline UInt8 readBooleanAligned() {
	return readUInt8aligned()!=0;
    }
    inline UInt8 readUInt8() {
	UInt8 r = *(UInt8 *)pos;
	pos+=sizeof(UInt8);
        return r;
    }
    inline UInt8 readUInt8aligned() {
	UInt8 r = *(UInt8 *)pos;
	pos+=2;
        return r;
    }
    inline UInt16 readUInt16() {
	UInt16 r = *(UInt16 *)pos;
	pos+=sizeof(UInt16);
        return r;
    }
    inline Int16 readInt16() {
	Int16 r = *(Int16 *)pos;
	pos+=sizeof(Int16);
        return r;
    }
    inline UInt32 readUInt32() {
	UInt32 r = *(UInt32 *)pos;
	pos+=sizeof(UInt32);
        return r;
    }
    inline Int32 readInt32() {
	Int32 r = *(Int32 *)pos;
	pos+=sizeof(Int32);
        return r;
    }
    inline UInt8 *getCurrentPos() {
        return pos;
    }
    inline void skip(UInt16 bytes) {
        pos+=bytes;
    }
    inline void readBlock(MemPtr out, UInt16 len) {
	MemMove(out, pos, len);
	pos+=len;
    }
    inline char *readString() {
        UInt16 len = readUInt16();
	char *res = (char *)pos;
	pos+=len+1;
        if (len%2==0) pos++;
        return res;
    }
};

class MemStreamWriter {
protected:
    UInt8 *stream;
    UInt16 ssize, allocated;
public:
    MemStreamWriter() {
	stream = NULL;
	ssize=0;
	allocated=0;
    }
    inline void writeBoolean(Boolean value) {
	writeUInt8(value ? 1:0);
    }
    inline void writeBooleanAligned(Boolean value) {
	writeUInt8aligned(value ? 1:0);
    }
    void writeUInt8(UInt8 value);
    void writeUInt8aligned(UInt8 value);
    void writeUInt16(UInt16 value);
    void writeUInt32(UInt32 value);
    void writeInt32(Int32 value);
    void writeBlock(MemPtr in, UInt16 len);
    void writeString(char *str);
    inline void clear() {
	if (stream != NULL) {
	    MemPtrFree(stream);
	    stream = NULL;
	    ssize = 0;
            allocated = 0;
	}
    }
    Err writeToDB(DmOpenRef ref, UInt16 *record);
    inline UInt16 size() {
        return ssize;
    }
};

#endif
