/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#ifndef apUtilsTemplates
#define apUtilsTemplates

#include <PalmOS.h>

Boolean UtilsTemplatesRealloc(void **data, UInt16& allocated, const UInt16 sz, const UInt8 sizeofT);

template <class Titem>
class Vector {
protected:
    UInt16 allocated,count;
    Titem *data;
    inline Boolean realloc(UInt16 sz) {
        return UtilsTemplatesRealloc(&(void *)data, allocated, sz, sizeof(Titem));
    }
public:
    Vector() {
	allocated = 0;
	count = 0;
        data = NULL;
    }
    inline ~Vector() {
        clear();
    }
    void clear() {
	if (data != NULL) {
	    MemPtrFree(data);
            data = NULL;
	}
	allocated = 0;
        count = 0;
    }
    inline void safeClear() {
        count = 0;
    }
    void add(Titem value) {
	if (allocated<=count+1)
            if (!realloc(count+1)) return;

	data[count] = value;
        count++;
    }
    inline void addVector(Vector<Titem>& vector) {
	add(vector.getArray(), vector.size());
    }
    void add(const Titem *values, UInt16 addCount) {
	if (allocated<=count+addCount)
            if (!realloc(count+addCount)) return;

	MemMove(data+count,values,sizeof(Titem)*addCount);
        count+=addCount;
    }
    void remove(UInt16 i) {
	if (i>=count) return;
	if (i<count-1) {
	    MemMove(data+i,data+i+1,sizeof(Titem)*(count-i-1));
	}
	count--;
    }
    UInt16 find(Titem value) {
	for (UInt16 i=0; i<count; i++)
	    if (data[i] == value)
		return i;
	return (UInt16)-1;
    }
    Titem get(UInt16 i) {
	if (i>=count) return (Titem)0;
	return data[i];
    }
    inline Titem *getArray() {
        return data;
    }
    void set(UInt16 i, Titem value) {
	if (i>=count) return;
	data[i] = value;
    }
    inline UInt16 size() {
        return count;
    }
};

UInt16 findStringInVector(Vector<char *>& vector, char *str);

template <class Titem>
class ArraySet {
protected:
    UInt16 allocated,count;
    Titem *data;
    inline Boolean realloc(UInt16 sz) {
        return UtilsTemplatesRealloc(&(void *)data, allocated, sz, sizeof(Titem));
    }
public:
    ArraySet() {
	allocated = 0;
	count = 0;
        data = NULL;
    }
    inline ~ArraySet() {
        clear();
    }
    void clear() {
	if (data != NULL) {
	    MemPtrFree(data);
            data = NULL;
	}
	allocated = 0;
        count = 0;
    }
    inline void safeClear() {
        count = 0;
    }
    void put(Titem value) {
	UInt16 i;
	if (count > 0) {
	    for (i=0; i<count; i++) {
		if (data[i] >= value) break;
	    }
	    if (data[i] == value) return;
	} else {
	    i=0;
	}

	if (allocated<=count+1)
	    if (!realloc(count+1)) return;

	if (count > 0) {
	    UInt16 countMove = count-i;
	    if (countMove > 0) {
		MemMove(&data[i+1], &data[i], countMove*sizeof(Titem));
	    }
	}
	data[i] = value;
        count++;
    }
    UInt16 find(Titem value);
    Titem get(UInt16 i) {
	if (i<0 || i>=count) return 0;
	return data[i];
    }
    inline Titem *getArray() {
        return data;
    }
    void set(UInt16 i, Titem value) {
	if (i<0 || i>=count) return;
	data[i] = value;
    }
    inline UInt16 size() {
        return count;
    }
};

template <class Titem>
class ArrayMap {
protected:
    Vector<UInt16> keys;
    Vector<Titem> values;
public:
    void put(UInt16 key, Titem value) {
        UInt16 i;
	for (i=0; i<keys.size(); i++) {
	    if (key == keys.get(i)) {
                values.set(i, value);
                return;
	    }
	}
	keys.add(key);
        values.add(value);
    }
    Titem get(UInt16 key) {
        UInt16 i;
	for (i=0; i<keys.size(); i++) {
	    if (key == keys.get(i)) {
                return values.get(i);
	    }
	}
	return NULL;
    }
};

#endif
