/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "AlarmEngine.h"
#include "TVdata.h"
#include "../ids.h"
#include "../UI/Details.h"

AlarmEngine::AlarmEngine(Core *pCore) {
    DmSearchStateType searchInfo;

    vCore=pCore;
    det = NULL;

    soundType[0]='\0';
    offset=0;
    DmGetNextDatabaseByTypeCreator(true, &searchInfo, sysFileTApplication,
                                   'abTV', true, &cardNo, &dbID);
}

void AlarmEngine::playAlarm() {
    playMidi(soundType);
}

void AlarmEngine::playMidi(Char *name) {
    UInt16 rc;
    MemHandle rl,han;
    void *p;
    UInt8 *data;
    SndMidiListItemType *rec;
    DmOpenRef dbref;
    UInt16 indexP;
    Boolean found=false;

    if (SndCreateMidiList(0,true,&rc,&rl)) {
	p=MemHandleLock(rl);
	for (UInt16 i=0; i<rc; i++) {
            rec=(SndMidiListItemType *)p+i;
	    if (StrCompare(name,rec->name)==0) {
		if ((dbref=DmOpenDatabase(rec->cardNo,rec->dbID,dmModeReadOnly))==NULL)
		    break;
		if (DmFindRecordByID(dbref,rec->uniqueRecID,&indexP)!=errNone)
                    break;
		if ((han=DmQueryRecord(dbref,indexP))==NULL)
		    break;
		data=(UInt8 *)MemHandleLock(han);
		SndPlaySmf(NULL,sndSmfCmdPlay,data,NULL,NULL,NULL,false);
                found=true;
                MemHandleUnlock(han);
	    }
	}
	MemHandleUnlock(rl);
    }
    if (!found)
	SndPlaySystemSound(sndAlarm);
}

void AlarmEngine::saveToStream(MemStreamWriter &stream) {
    stream.writeInt32(offset);
    stream.writeBlock((UInt8 *)soundType,sizeof(soundType));
}

void AlarmEngine::loadFromStream(MemStreamReader &stream) {
    offset=stream.readInt32();
    stream.readBlock((MemPtr)soundType,sizeof(soundType));
}

void AlarmEngine::setOffset(UInt32 value) {
    offset=value;
}

static void AlarmProc(UInt16 almProcCmd, SysAlarmTriggeredParamType *paramP) {
    if (almProcCmd == almProcCmdTriggered) {
	AlarmEngine *alm = (AlarmEngine *)paramP->ref;
	alm->playAlarm();
	alm->displaySetData(paramP->alarmSeconds);
    }
}

void AlarmEngine::setNextAlarm(Boolean internal) {
    UInt16 tmchannel=0;
    UInt32 time = 0; // time for alarm(GMT)
    UInt32 alarmTime = 3;

    if (isOn()) { // find alarm
	time=vCore->vTVdata->selectminTime(TimGetSeconds()-vCore->vTVdata->coreGMToff+getOffset(),&tmchannel);
	if (time != 0) {
	    alarmTime = time+vCore->vTVdata->coreGMToff-getOffset();
	}
    }

    if (time == 0) {
        clearNextAlarm(internal);
    } else {
	if (internal) {
            AlmSetProcAlarm(AlarmProc, (UInt32)this, alarmTime);
	} else {
	    AlmSetAlarm(cardNo, dbID, 0, alarmTime, true);
	}
    }
}

void AlarmEngine::clearNextAlarm(Boolean internal) {
    if (internal) {
	AlmSetProcAlarm(AlarmProc, 0, 0);
    } else {
        AlmSetAlarm(cardNo, dbID, 0, 0, true);
    }
}

void AlarmEngine::displaySetData(UInt32 time) {
    det->program = vCore->vTVdata->findProgramByGMTTime(time-vCore->vTVdata->coreGMToff+getOffset(),
						       &(det->channel));
    setNextAlarm(true);
}

void AlarmEngine::display(UInt32 time) {
    det=new Details(vCore);
 HostFPrintF(HostLogFile(),"alarm 1");

    displaySetData(time);
 HostFPrintF(HostLogFile(),"alarm 2");
    det->process();
 HostFPrintF(HostLogFile(),"alarm 3");
    clearNextAlarm(true);
 HostFPrintF(HostLogFile(),"alarm 4");

    delete det;
    det = NULL;
}

Err myWakeupProc(SysNotifyParamType *p) {
    if (core->vFilter->needCalc() && (FrmGetActiveFormID()==idFormMain)) {
        core->vMainTable->refreshData();
    }
    return errNone;
}
