/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.4 $ 
 *     $Date: 2003/10/24 14:24:07 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "Core.h"

Core *core;

#define DBERROR      {e=DmGetLastErr(); if (e!=errNone) {ErrAlert(e); return;}}
#define DBKNOWNERROR {if (e!=errNone) {ErrAlert(e); return;}}

static void switchToNewScreenMode() {
    if (core->vMainTable == NULL) return;

    core->setScreenBound();
    core->vMainTable->changeSize(true);
}

static Err HandleNotificationDisplayChange(SysNotifyParamType *cmdPBP) {
    switchToNewScreenMode();
}

static void registerNotify() {
    if (isSilkOn()) {
	Err e;
	if ((e=SysNotifyRegister(0,0,sysNotifyDisplayChangeEvent,
			     HandleNotificationDisplayChange,
			     sysNotifyNormalPriority, NULL)) != errNone)
	ErrAlert(e);
    }
}
static void unregisterNotify() {
    if (isSilkOn()) {
	Err e;
	if ((e=SysNotifyUnregister(0,0,sysNotifyDisplayChangeEvent,
				   sysNotifyNormalPriority)) != errNone)
	    ErrAlert(e);
    }
}

void Core::setScreenBound() {
    Coord width, height;

    AWinGetDisplayExtent(&width,&height);
    RectangleType rect;
    rect.topLeft.x = 0;
    rect.topLeft.y = 0;
    rect.extent.x = width;
    rect.extent.y = height;
    AWinSetWindowBounds(WinGetActiveWindow(),&rect);
}

Core::Core(Boolean engineOnly) {
    Err e;

    vMainTable = NULL;

    if (!engineOnly) {
	core = this;
	needSave = true;
	HiResModeStart();
	if (HideSilkStart()) {
            registerNotify();
	    switchToNewScreenMode();
	}
    } else {
	needSave = false;
    }
    vTVdata = new TVdata(this, engineOnly);

    vLocale = new Locale();
    if (!engineOnly) {
	vFilter = new Filter(vTVdata);
    } else {
        vFilter = NULL;
    }
    vAlarmEngine = new AlarmEngine(this);

    FONTbase = 0;

    prefOpen();

    if (!engineOnly) {
	if ((e=SysNotifyRegister(0,0,sysNotifyLateWakeupEvent,myWakeupProc,sysNotifyNormalPriority,NULL))!=errNone) {
	    ErrAlert(e);
	}
        notifyRegistered = true;
    } else {
        notifyRegistered = false;
    }
}

Core::~Core() {
    Err e;

    if (notifyRegistered) {
	if ((e=SysNotifyUnregister(0,0,sysNotifyLateWakeupEvent,sysNotifyNormalPriority))!=errNone) {
	    ErrAlert(e);
	}
    }

    if (needSave) {
	prefClose();
    }

    delete vAlarmEngine;

    if (vMainTable != NULL) {
        delete vMainTable;
    }
    if (vFilter != NULL) {
	delete vFilter;
    }
    delete vTVdata;
    delete vLocale;

    if (needSave) {
        unregisterNotify();
	HideSilkStop();
	HiResModeStop();
    }
}

void Core::prefOpen() {
    Err e;

    UInt16 card;
    LocalID currentDB = 0;
    UInt16 ver;
    DmSearchStateType state;

    e = DmGetNextDatabaseByTypeCreator(true,&state,'pref','abTV',true,&card,&currentDB);
    if (e) return;
    // check version
    e=DmDatabaseInfo(card,currentDB,NULL,NULL,&ver,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
    DBKNOWNERROR
    if (ver==VERSIONpref) {
        prefOpenCurrent(card, currentDB);
    }
}

void Core::prefOpenCurrent(UInt16 card, LocalID currentDB) {
    Err e;
    DmOpenRef prefDB=NULL;
    MemHandle handle;

    if ((prefDB=DmOpenDatabase(card,currentDB,dmModeReadOnly))==NULL) DBERROR;
    if (DmNumRecords(prefDB)==4) {
	if (vTVdata!=NULL) {
	    handle=DmQueryRecord(prefDB,0); DBERROR;
	    MemStreamReader in(MemHandleLock(handle));
	    vTVdata->loadFromStream(in);
	    MemHandleUnlock(handle);
	}

	if (vFilter!=NULL) {
	    handle=DmQueryRecord(prefDB,1); DBERROR;
	    MemStreamReader in(MemHandleLock(handle));
	    vFilter->loadFromStream(in);
	    MemHandleUnlock(handle);
	}

	if (vAlarmEngine!=NULL) {
	    handle=DmQueryRecord(prefDB,2); DBERROR;
	    MemStreamReader in(MemHandleLock(handle));
            vAlarmEngine->loadFromStream(in);
	    MemHandleUnlock(handle);
	}

	handle=DmQueryRecord(prefDB,3); DBERROR;
	MemStreamReader in(MemHandleLock(handle));
        loadFromStream(in);
	MemHandleUnlock(handle);
    }
    DmCloseDatabase(prefDB);
}

void Core::loadFromStream(MemStreamReader& stream) {
    FONTbase = stream.readUInt8aligned();
}

void Core::saveToStream(MemStreamWriter& stream) {
    stream.writeUInt8aligned(FONTbase);
}

void Core::prefClose() {
    Err e;
    UInt16 index;
    DmOpenRef prefDB=NULL;
    UInt16 card;
    LocalID currentDB = 0;
    DmSearchStateType state;
    UInt16 ver;

    while (true) {
        e = DmGetNextDatabaseByTypeCreator(true,&state,'pref','abTV',true,&card,&currentDB);
        if (e!=errNone) break;
        DmDeleteDatabase(card,currentDB);
    }
    e=DmCreateDatabase(0,"TV Guide preferences",'abTV','pref',false);
    DBKNOWNERROR;
    e = DmGetNextDatabaseByTypeCreator(true,&state,'pref','abTV',true,&card,&currentDB);
    DBKNOWNERROR;
    ver = VERSIONpref;
    e = DmSetDatabaseInfo(card,currentDB,NULL,NULL,&ver,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
    DBKNOWNERROR;

    if ((prefDB=DmOpenDatabase(card,currentDB,dmModeWrite | dmModeExclusive))==NULL) DBERROR;
    MemStreamWriter out;

    vTVdata->saveToStream(out);
    index=dmMaxRecordIndex;
    e=out.writeToDB(prefDB,&index);
    DBKNOWNERROR;
    out.clear();

    vFilter->saveToStream(out);
    index=dmMaxRecordIndex;
    e=out.writeToDB(prefDB,&index);
    DBKNOWNERROR;
    out.clear();

    vAlarmEngine->saveToStream(out);
    index=dmMaxRecordIndex;
    e=out.writeToDB(prefDB,&index);
    DBKNOWNERROR;
    out.clear();

    saveToStream(out);
    index=dmMaxRecordIndex;
    e=out.writeToDB(prefDB,&index);
    DBKNOWNERROR;
    out.clear();

    DmCloseDatabase(prefDB);
}

void Core::reCreateMainTable() {
    delete vMainTable;
    vMainTable=new MainTable(FrmGetActiveForm(), this);
    vMainTable->refreshData();
}
