/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "Filter.h"
#include "../ids.h"
#include "TVdata.h"
#include "TVdataUtils.h"
#include "Core.h"

Filter::Filter(TVdata *pTVdata) {
    vTVdata = pTVdata;
    NowForward=3L*60L*60L;
    DayBegin=(5L*60L+55L)*60L;
    ViewChan=1;
    ViewSort=0;
    Mode=0;
    Selected=0;
    currentGroup = (UInt16)-1;
}

Filter::~Filter() {
    while (groupNames.size()>0) {
	delGroup(0);
    }
}

void Filter::del() {
    while (groupNames.size()>0) {
        delGroup(0);
    }
    currentGroup = (UInt16) -1;
}

UInt16 Filter::rowChannel(UInt16 row) {
    return programs.get(row)->channel;
}
UInt8 *Filter::rowProgram(UInt16 row) {
    return programs.get(row)->prog;
}

Boolean Filter::needCalc() {
    return Mode==0 || Mode==1;
}

void Filter::recalc() {
    switch (Mode) {
    case 0:
        DateFrom=TimGetSeconds();
        DateTo=DateFrom+NowForward;
        break;
    case 1:
        DateFrom=TimGetSeconds();
        DateTo=(DateFrom/(24L*60*60)+1)*24L*60*60+DayBegin;
        break;
    case 2:
	DateFrom=0;
        DateTo=0;
        break;
    case 3:
       	DateFrom=Day*24L*60*60+DayBegin;
        DateTo=DateFrom+24L*60*60;
        break;
    }
    makeList();
}

void Filter::performRecord(UInt8 *prog,UInt32 chanid) {
    if (DateFrom>0) {
	if (vTVdata->Program_getBegTime(prog) >= DateTo) return;
	if (vTVdata->Program_getBegTime(prog)+Program_getLength(prog) < DateFrom) return;
    }

    switch (Selected) {
    case 0:
        break;
    case 1:
        if (!(vTVdata->program_isChecked(prog,chanid)))
            return;
        break;
    case 2:
        if (vTVdata->program_isChecked(prog,chanid))
            return;
        break;
    }
    struct ProgData data;
    if (currentChannel!=chanid && ViewChan && !ViewSort) {
	// add channel header
	data.prog=NULL;
        data.channel=chanid;
        programs.add(data);
        currentChannel=chanid;
    }
    data.prog=prog;
    data.channel=chanid;
    programs.add(data);
}

static Int16 compare(void *s1,void *s2,Int32 pFilter) {
    Filter *vFilter = (Filter *)pFilter;

    UInt32 time1 = vFilter->vTVdata->Program_getBegTime(((struct ProgData *)s1)->prog);
    UInt32 time2 = vFilter->vTVdata->Program_getBegTime(((struct ProgData *)s2)->prog);

    if (time1==time2) return 0;
    if (time1>time2)
        return 1;
    else
        return -1;
}

void Filter::makeList() {
    programs.safeClear();

    currentChannel=(UInt32)-1;
    filterAllChannels();
    if (ViewSort && programs.size()>1) {
	SysQSort(programs.getArray(),programs.size(),sizeof(struct ProgData),&compare,(Int32)this);
    }
}

Boolean Filter::allowDisplayChannel(UInt16 chanid) {
    if (currentGroup==(UInt16)-1) {
        return true;
    } else {
	Vector<UInt16> *list=groupIncludes.get(currentGroup);
	return list->find(chanid) != (UInt16)-1;
    }
}

void Filter::filterAllChannels() {
    for (UInt16 chanId=0; chanId<vTVdata->channelNames.size(); chanId++) {
	if (!allowDisplayChannel(chanId)) continue;
        vTVdata->filterOneChannel(chanId, this);
    }
}


UInt32 Filter::getRows() {
    return programs.size();
}

void Filter::loadFromStream(MemStreamReader& stream) {
    UInt16 maxi, i, maxj, j;
    Char *gname, *cname;
    UInt32 cid;
    Vector<UInt16> gincs;

    NowForward=stream.readUInt32();
    DayBegin=stream.readUInt32();
    ViewChan=stream.readUInt8aligned();
    ViewSort=stream.readUInt8aligned();
    Selected=stream.readUInt8aligned();
    Mode=stream.readUInt16();
    Day=stream.readUInt32();
    currentGroup=stream.readUInt16();

    // load group list
    maxi=stream.readInt16();
    for (i=0; i<maxi; i++) {
        gname = stream.readString();
	maxj=stream.readUInt16();
	for(j=0; j<maxj; j++) {
	    cname=stream.readString();
	    cid = vTVdata->getChannelID(cname);
	    if (cid != (UInt16)-1) {
                gincs.add(cid);
	    }
	}
	newGroup(gname, gincs);
        gincs.clear();
    }
}

void Filter::saveToStream(MemStreamWriter& stream) {
    UInt16 i,j;

    stream.writeUInt32(NowForward);
    stream.writeUInt32(DayBegin);
    stream.writeUInt8aligned(ViewChan);
    stream.writeUInt8aligned(ViewSort);
    stream.writeUInt8aligned(Selected);
    stream.writeUInt16(Mode);
    stream.writeUInt32(Day);
    stream.writeUInt16(currentGroup);

    // save group list
    stream.writeUInt16(groupNames.size());
    for (i=0; i<groupNames.size(); i++) {
	stream.writeString(groupNames.get(i));
        Vector<UInt16> *data = groupIncludes.get(i);
	stream.writeUInt16(data->size());
	for (j=0; j<data->size(); j++) {
	    stream.writeString(vTVdata->getChannelName(data->get(j)));
	}
    }
}

UInt16 Filter::newGroup(Char *name, Vector<UInt16>& channels) {
    UInt16 groupId = groupNames.size();

    Vector<UInt16> *data = new Vector<UInt16>;

    Char *namecopy = (Char *)MemPtrNew(StrLen(name)+1);
    StrCopy(namecopy,name);

    data->addVector(channels);

    groupNames.add(namecopy);
    groupIncludes.add(data);

    return groupId;
}

void Filter::delGroup(UInt16 groupId) {
    Vector<UInt16> *data = groupIncludes.get(groupId);
    delete data;

    MemPtrFree(groupNames.get(groupId));
    groupNames.remove(groupId);
    groupIncludes.remove(groupId);
}
