/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "../ids.h"
#include "Locale.h"

Locale::Locale() {
    sResMHAll=DmGetResource('tSTR',sAll);
    sResMPAll=(Char *)MemHandleLock(sResMHAll);
    sResMHNow=DmGetResource('tSTR',sNow);
    sResMPNow=(Char *)MemHandleLock(sResMHNow);
    sResMHToday=DmGetResource('tSTR',sToday);
    sResMPToday=(Char *)MemHandleLock(sResMHToday);
    sResMHGroupNew=DmGetResource('tSTR',sGroupNew);
    sResMPGroupNew=(Char *)MemHandleLock(sResMHGroupNew);
    sResMHGroupEdit=DmGetResource('tSTR',sGroupEdit);
    sResMPGroupEdit=(Char *)MemHandleLock(sResMHGroupEdit);
    sResMHSel=DmGetResource('tSTR',sSel);
    sResMPSel=(Char *)MemHandleLock(sResMHSel);
    sResMHUns=DmGetResource('tSTR',sUns);
    sResMPUns=(Char *)MemHandleLock(sResMHUns);
    sResMHNone=DmGetResource('tSTR',sNone);
    sResMPNone=(Char *)MemHandleLock(sResMHNone);

    MemSet(daysname,sizeof(daysname),0);
    for (int i=0; i<7; i++)
        SysStringByIndex(stDays,i,daysname[i],4);
}

void Locale::destructor() {
    MemHandleUnlock(sResMHAll);
    DmReleaseResource(sResMHAll);
    MemHandleUnlock(sResMHNow);
    DmReleaseResource(sResMHNow);
    MemHandleUnlock(sResMHToday);
    DmReleaseResource(sResMHToday);
    MemHandleUnlock(sResMHGroupNew);
    DmReleaseResource(sResMHGroupNew);
    MemHandleUnlock(sResMHGroupEdit);
    DmReleaseResource(sResMHGroupEdit);
    MemHandleUnlock(sResMHSel);
    DmReleaseResource(sResMHSel);
    MemHandleUnlock(sResMHUns);
    DmReleaseResource(sResMHUns);
    MemHandleUnlock(sResMHNone);
    DmReleaseResource(sResMHNone);
}

Char *Locale::dayname(UInt8 num) {
    return daysname[num];
}

Char *Locale::timeToStr(UInt32 time) { // seconds to string
    switch (time) {
    case 0: return sResMPNone;
    case 10: return "10s";
    case 30: return "30s";
    case 60: return "1m";
    case 180: return "3m";
    case 300: return "5m";
    case 600: return "10m";
    case 900: return "15m";
    case 1800: return "30m";
    case  60*60: return "1h";
    case  90*60: return "1h 30m";
    case 120*60: return "2h";
    case 180*60: return "3h";
    case 240*60: return "4h";
    case 300*60: return "5h";
    }
    return sResMPNone;
}

UInt32 Locale::strToTime(Char *str) {
    if (StrCompare(str,sResMPNone) == 0) {
        return 0;
    } else if (StrCompare(str,"10s") == 0) {
        return 10;
    } else if (StrCompare(str,"30s") == 0) {
        return 30;
    } else if (StrCompare(str,"1m") == 0) {
        return 60;
    } else if (StrCompare(str,"3m") == 0) {
	return 180;
    } else if (StrCompare(str,"5m") == 0) {
        return 300;
    } else if (StrCompare(str,"10m") == 0) {
        return 600;
    } else if (StrCompare(str,"15m") == 0) {
        return 900;
    } else if (StrCompare(str,"30m") == 0) {
        return 1800;
    } else if (StrCompare(str,"1h") == 0) {
        return 60*60;
    } else if (StrCompare(str,"1h 30m") == 0) {
        return 90*60;
    } else if (StrCompare(str,"2h") == 0) {
        return 120*60;
    } else if (StrCompare(str,"3h") == 0) {
        return 180*60;
    } else if (StrCompare(str,"4h") == 0) {
        return 240*60;
    } else if (StrCompare(str,"5h") == 0) {
        return 300*60;
    }
    return 0;
}
