/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.3 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "TVdata.h"
#include "../ids.h"
#include "../UI/FontSel.h"
#include "TVdataUtils.h"
#include "Filter.h"
#include "Core.h"
#include "../APalmLib/UtilsTemplates.h"

Boolean TVdata::needToDelete(DmOpenRef db) {
    Err e;
    MemHandle m;
    UInt8 *p;
    UInt32 firstTime=TimGetSeconds()-10L*24L*60*60; // allow 10 days for store program, local time


    if (DmNumRecords(db)<=0)
	return true;

    Boolean old=true;
    for (UInt32 i=0; i<DmNumRecords(db); i++) {
	m=DmQueryRecord(db,i);
	if (m==0) {
	    Err e=DmGetLastErr();
	    ErrAlert(e);
	}
	p=(UInt8 *)MemHandleLock(m);
	p=Program_passStr0(p);
	Int32 count=*(Int32 *)p; p+=sizeof(Int32);
	for (Int32 i=0; i<count; i++) {
	    UInt32 time=Program_getBegTime(p);
	    if (time>firstTime) {
		old=false;
		break;
	    }
	    p=Program_passProgram(p);
	}
	MemHandleUnlock(m);
    }
    return old;
}

TVdata::TVdata(Core *pCore, Boolean isAlarm) {
    vCore = pCore;
    fromAlarm=isAlarm;

    detectGMT=true;
    coreGMToff = 0;

    loadData();
}

TVdata::~TVdata() {
    unloadData();
}

static Int16 cmpDate(void *p1,void *p2,Int32 other) {
    return *(UInt32 *)p1-*(UInt32 *)p2;
}
static Int16 cmpString(void *first,void *second, Int32 other) {
    return StrCompare(*(Char **)first,*(Char **)second);
}

void TVdata::loadData() {
    Err e;
    DmSearchStateType state;
    Boolean latestVer=false;
    UInt16 card;
    LocalID currentDB = 0;
    DmOpenRef db;
    UInt16 ver;
    MemHandle m;
    UInt16 n;
    UInt8 *p;

    e = DmGetNextDatabaseByTypeCreator(true,&state,'Data','abTV',latestVer,&card,&currentDB);
    while (!e && currentDB) {
        // check version
        e=DmDatabaseInfo(card,currentDB,NULL,NULL,&ver,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
        if (e!=errNone) {ErrAlert(e); return;}

        if (ver==VERSIONdata) {
            if ((db=DmOpenDatabase(card,currentDB,dmModeReadOnly))==NULL) {
                e=DmGetLastErr();
                ErrAlert(e);
                return;
	    }
	    Boolean needDelete = false;
	    if (!fromAlarm) {
		if (needToDelete(db))
                    needDelete = true;
	    }
	    if (needDelete) {
		DmCloseDatabase(db);
		DmDeleteDatabase(card,currentDB);
	    } else {
		dbs.add(db);
		for (UInt32 i=0; i<DmNumRecords(db); i++) {
		    m=DmQueryRecord(db,i);
		    if (m==0) {
			Err e=DmGetLastErr();
			ErrAlert(e);
		    }
		    p=(UInt8 *)MemHandleLock(m);
		    n=findStringInVector(channelNames,(Char *)(p+sizeof(UInt16)));
		    if (n==(UInt16)-1)
			channelNames.add((char *)(p+sizeof(UInt16)));
		    RecordHandles.add(m);
		    RecordPointers.add(p);

		    if (!fromAlarm) {
			// collate dates
			p=Program_passStr0(p);
			Int32 count=*(Int32 *)p; p+=sizeof(Int32);
			for (Int32 i=0; i<count; i++) {
			    UInt32 day=Program_getBegTime(p)/60/60/24;
			    if (dates.find(day)==(UInt16)-1)
				dates.add(day);
			    p=Program_passProgram(p);
			}
		    }
		}
	    }
        } else {
            DmDeleteDatabase(card,currentDB);
        }
        e=DmGetNextDatabaseByTypeCreator(false,&state,'Data','abTV',latestVer,&card,&currentDB);
    }
    if (!fromAlarm) {
	if (dbs.size()==0 && !fromAlarm) {
	    FrmAlert(idAlertNoData);
	}

	// arrange channel names
	SysQSort(channelNames.getArray(),channelNames.size(),
		 sizeof(Char *),cmpString,0);

        // arrange dates
	SysQSort(dates.getArray(),dates.size(),sizeof(UInt32),cmpDate,0);
    }

    for (UInt32 i=0; i<RecordPointers.size(); i++) {
        p=RecordPointers.get(i);
	n=findStringInVector(channelNames,(Char *)(p+sizeof(UInt16)));
        COR.add(n);
    }

    for (UInt32 i=0; i<channelNames.size(); i++)
        channelChecks.add(new Vector<UInt32>);
}

void TVdata::unloadData() {
    for (UInt32 i=0; i<RecordHandles.size(); i++)
        MemHandleUnlock(RecordHandles.get(i));

    for (UInt32 i=0; i<channelChecks.size(); i++)
	delete channelChecks.get(i);

    for (UInt32 i=0; i<dbs.size(); i++)
        DmCloseDatabase(dbs.get(i));
}

void TVdata::del() {
    DmSearchStateType state;
    Boolean latestVer=false;
    UInt16 card;
    LocalID currentDB = 0;
    Err e;

    unloadData();
    while (true) {
        e = DmGetNextDatabaseByTypeCreator(true,&state,'Data','abTV',latestVer,&card,&currentDB);
        if (e!=errNone || currentDB==0) break;
        if ((e=DmDeleteDatabase(card,currentDB))!=errNone) {
            ErrAlert(e);
            break;
        }
    }
}

void TVdata::program_setChecked(UInt8 *prog,UInt16 chanid,Boolean value) {
    Vector<UInt32> *chdata=channelChecks.get(chanid);
    UInt32 gmtTime=Program_getBegTimeGMT(prog);
    UInt32 pos=chdata->find(gmtTime);
    if (pos!=(UInt16)-1 && value==false)
        chdata->remove(pos);
    if (pos==(UInt16)-1 && value==true)
        chdata->add(gmtTime);
}

Boolean TVdata::program_isChecked(UInt8 *prog,UInt16 chanid) {
    Vector<UInt32> *chdata=channelChecks.get(chanid);
    return chdata->find(Program_getBegTimeGMT(prog))!=(UInt16)-1;
}

UInt32 TVdata::selectminTime(UInt32 mintime,UInt16 *channelnum) {
    UInt32 tm,mintm;
    Vector<UInt32> *chdata;

    mintm=0;
    for (UInt16 i=0; i<channelNames.size(); i++) {
        chdata=channelChecks.get(i);
        for (UInt32 j=0; j<chdata->size(); j++) {
            tm=chdata->get(j); // alarm time
	    if (tm>mintime && (tm<mintm || mintm==0)) {
                mintm=tm;
                *channelnum=i;
            }
        }
    }
    return mintm;
}

UInt8 *TVdata::findProgramByGMTTime(UInt32 time, UInt16 *channel) {
    for (UInt16 realRecord=0; realRecord<RecordPointers.size(); realRecord++) {
        UInt16 chanid = COR.get(realRecord);
        UInt8 *rec = RecordPointers.get(realRecord);

        rec=Program_passStr0(rec);
        Int32 count=*(Int32 *)rec; rec+=sizeof(Int32);
        for (Int32 i=0; i<count; i++) {
            if (Program_getBegTimeGMT(rec)==time) {
		if (program_isChecked(rec,chanid)) {
		    *channel = chanid;
                    return rec;
		}
	    }
            rec=Program_passProgram(rec);
        }
    }
    return NULL;
}

void TVdata::loadFromStream(MemStreamReader& stream) {
    UInt16 i,j,maxi,maxj;
    Char *cname; UInt16 cnum;
    Vector<UInt32> *vidata;
    UInt32 readedData;

    detectGMT=stream.readBooleanAligned();
    coreGMToff=stream.readInt32();

    // load selected values
    maxi=stream.readUInt16();
    for (i=0; i<maxi; i++) {
        cname=stream.readString();
	if (cname==NULL) break;

        cnum=getChannelID(cname);
	if (cnum!=(UInt16)-1) {
            vidata=channelChecks.get(cnum);
	} else {
            vidata=NULL;
	}

        maxj=stream.readUInt16();
        for(j=0; j<maxj; j++) {
            readedData = stream.readUInt32();
	    if (vidata!=NULL) {
                vidata->add(readedData);
	    }
        }
    }
}

void TVdata::saveToStream(MemStreamWriter& stream) {
    stream.writeBooleanAligned(detectGMT);
    stream.writeInt32(coreGMToff);

    // add selected values
    stream.writeUInt16(channelNames.size());
    for (UInt16 i=0; i<channelNames.size(); i++) {
        stream.writeString(channelNames.get(i));
        Vector<UInt32> *vidata=channelChecks.get(i);
        if (vidata==NULL) {
	    stream.writeUInt16(0);
        } else {
            stream.writeUInt16(vidata->size());
            for (UInt32 j=0; j<vidata->size(); j++) {
                stream.writeUInt32(vidata->get(j));
            }
        }
    }
}

UInt16 TVdata::getChannelID(Char *cname) {
    return findStringInVector(channelNames,cname);
}

Char *TVdata::getChannelName(UInt16 chanId) {
    return channelNames.get(chanId);
}

void TVdata::filterOneChannel(UInt16 filterChanId, Filter *vFilter) {
    for (UInt16 realRecord=0; realRecord<RecordPointers.size(); realRecord++) {
	UInt32 chanId=COR.get(realRecord);
        if (chanId != filterChanId) continue;

	UInt8 *rec=RecordPointers.get(realRecord);
	rec=Program_passStr0(rec);
	Int32 count=*(Int32 *)rec; rec+=sizeof(Int32);
	for (Int32 i=0; i<count; i++) {
	    vFilter->performRecord(rec,chanId);
	    rec=Program_passProgram(rec);
	}
    }
}


/*void TVdata::setGMT(Int32 value) {
    Int32 gmt;
    
    gmtvalue=value;

    if (gmtvalue==99) { // autodetect
        if (PrefGetPreference(prefVersion)<9) {
	    gmt=0;
	} else {
	    gmt=PrefGetPreference(prefTimeZone)*60+PrefGetPreference(prefDaylightSavingAdjustment)*60;
	}
    } else {
	gmt=gmtvalue;
    }

    coreGMToff=gmt;
} */
