/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.3 $ 
 *     $Date: 2003/10/24 14:21:37 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "../ids.h"
#include "../Engine/TVdata.h"
#include "Channels.h"
#include "../APalmLib/HiRes.h"
#include "../Engine/Filter.h"

chanTable::chanTable(FormPtr form, TVdata *pTVdata, Core *pCore) {
    vTVdata = pTVdata;
    if (isHiResOn())
	AFntSetFont(1);
    else
	AFntSetFont(0);
    init(form,idChanList,idChanScroll);
    Coord len2=rowHeight+2;
    Coord len1=pos.extent.x-len2;
    addColumn(len1);
    addColumn(len2);
    if (isHiResOn()) {
	bon=new Bitmap(idbmpcbon14x1);
	boff=new Bitmap(idbmpcboff14x1);
    } else {
	bon=new Bitmap(idbmpcbon11x1);
	boff=new Bitmap(idbmpcboff11x1);
    }
}

chanTable::~chanTable() {
    delete bon;
    delete boff;
}

void chanTable::drawCell(Int16 row,Int16 col,RectangleType *rect) {
    Char *chn;
    UInt16 chl;

    AWinEraseRectangle(rect,0);
    switch (col) {
    case 0:
        chn=vTVdata->channelNames.get(row);
        chl=StrLen(chn);
        AWinDrawTruncChars(chn,chl,rect);
        break;
    case 1:
        if (tempSeeList.find(row)!=(UInt16)-1)
            bon->draw(rect->topLeft.x,rect->topLeft.y);
        else
            boff->draw(rect->topLeft.x,rect->topLeft.y);
        break;
    }
}

void chanTable::selectCell(Int16 row,Int16 col) {
    if (col==1) { // checkbox
	UInt16 pos=tempSeeList.find(row);
        if (pos==(UInt16)-1)
            tempSeeList.add(row);
        else
            tempSeeList.remove(pos);
    }
    redrawCell(row,col);
}


Int16 chanTable::getRowCount() {
    return vTVdata->channelNames.size();
}

void chanTable::addChannels(Vector<UInt16>& channels) {
    tempSeeList.addVector(channels);
}


DlgChannels::DlgChannels(Core *pCore, UInt16 groupId) {
    vCore = pCore;
    vGroupId = groupId;
    nameH = MemHandleNew(NAMELEN_MAX);
    init(idFormChannels);
    list=new chanTable(frmP, vCore->vTVdata, pCore);
}

DlgChannels::~DlgChannels() {
    MemHandleFree(nameH);
    delete list;
}

void DlgChannels::load() {
    FieldPtr fldname = (FieldPtr)ObjectByID(idChanName);
    Char *namebuffer = (Char *)MemHandleLock(nameH);

    if (vGroupId != (UInt16)-1) {
	StrNCopy(namebuffer,
		 vCore->vFilter->groupNames.get(vCore->vFilter->currentGroup),
		 NAMELEN_MAX-1);
	list->addChannels(*(vCore->vFilter->groupIncludes.get(vGroupId)));
    } else {
        namebuffer[0] = '\0';
    }
    MemHandleUnlock(nameH);

    FldSetTextHandle(fldname, nameH);
}

void DlgChannels::save(UInt16 exitControl) {
    FieldPtr fldname = (FieldPtr)ObjectByID(idChanName);
    Char *namebuffer = (Char *)MemHandleLock(nameH);

    switch (exitControl) {
    case idChanBtnOK:
	if (vGroupId != (UInt16)-1) {
	    vCore->vFilter->delGroup(vGroupId);
	}
	vGroupId = vCore->vFilter->newGroup(namebuffer,list->tempSeeList);
        break;
    case idChanBtnCancel:
        break;
    case idChanBtnDelete:
	if (vGroupId != (UInt16)-1) {
	    vCore->vFilter->delGroup(vGroupId);
	}
        vGroupId = (UInt16)-1;
        break;
    }
    MemHandleUnlock(nameH);

    FldSetTextHandle(fldname, NULL);
}

Boolean DlgChannels::eventHandler(EventPtr eventP) {
    switch (eventP->eType) {
    case winExitEvent:
        list->redraw(true);
	return true;
    }
    return list->event(eventP);
}
