/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "../ids.h"
#include "Details.h"

Details::Details(Core *pCore) {
    vCore = pCore;
    outtext = NULL;
    program = NULL;
    frmP=FrmInitForm(idFormDetails);
}

Details::~Details() {
    FrmDeleteForm(frmP);
}

void *Details::ObjectByID(UInt16 ID) {
    return FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,ID));
}

static ScrollBarPtr getBar() {
    FormPtr frmP;

    frmP=FrmGetFormPtr(idFormDetails);
    return (ScrollBarPtr)FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,idScrollDetails));
}

static FieldPtr getText() {
    FormPtr frmP;

    frmP=FrmGetFormPtr(idFormDetails);
    return (FieldPtr)FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,idDetailsProg));
}

static void updateScroll(Int16 linesToScroll) {
    UInt16 scrollPos,textHeight,fieldHeight,blankLines;
    Int16 maxValue;

    FieldPtr fldtext=getText();
    blankLines=FldGetNumberOfBlankLines(fldtext);
    
    if (linesToScroll<0)
        FldScrollField(fldtext,-linesToScroll,winUp);
    else if (linesToScroll>0)
        FldScrollField(fldtext,linesToScroll,winDown);

    FldGetScrollValues(fldtext,&scrollPos,&textHeight,&fieldHeight);
    if (textHeight>fieldHeight)
        maxValue=(textHeight-fieldHeight)+blankLines;
    else if (scrollPos)
        maxValue=scrollPos;
    else
        maxValue=0;
    SclSetScrollBar(getBar(),scrollPos,0,maxValue,fieldHeight-1);
}

void Details::loadData() {
    UInt16 l1,l2;
    DateType dt;
    Int16 dow;
    Char *p1,*p2;

    FieldPtr fp2 = (FieldPtr)ObjectByID(idDetailsTime);
    FieldPtr fldtext = (FieldPtr)ObjectByID(idDetailsProg);

    // we should have equals size for channels and programs array !
    if (program != NULL) {
        // begin and end time
	UInt8 *pData = program;
        UInt16 pCh = channel;

	UInt32 beginTime=vCore->vTVdata->Program_getBegTime(pData);
	int mBegTime=beginTime/60%(24L*60);
        int mEndTime=(beginTime+Program_getLength(pData))/60%(24L*60);
        DateDaysToDate(beginTime/60/60/24,&dt);
        dow=(DayOfWeek(dt.month,dt.day,dt.year+1904)+6)%7;
	StrPrintF(timetext,"%2i:%02i-%2i:%02i   %i.%02i, %s",
		  mBegTime/60,mBegTime%60,
		  mEndTime/60,mEndTime%60,
		  dt.day,dt.month,
		  vCore->vLocale->dayname(dow));
        FldSetTextPtr(fp2,timetext);

        // name and announce
	p1=Program_getName(pData,&l1);
        p2=Program_getAnn(pData,&l2);
	if (outtext!=NULL) {MemPtrFree(outtext);}
        outtext=(Char *)MemPtrNew(l1+1+l2+1);
        StrNCopy(outtext,p1,l1);
        outtext[l1]='\n';
        StrNCopy(outtext+l1+1,p2,l2);
        outtext[l1+1+l2]='\0';

        FldSetTextPtr(fldtext,outtext); FldRecalculateField(fldtext,true);
        FrmSetTitle(frmP,vCore->vTVdata->channelNames.get(pCh));
        updateScroll(0);
    } else {
        FrmSetTitle(frmP,"Time to see TV !");
    }
}

void Details::close() {
    if (outtext!=NULL)
        MemPtrFree(outtext);
}

static Boolean detailsEventHandler(EventPtr eventP) {
    Int16 vl;

    switch (eventP->eType) {
    case sclRepeatEvent:
        updateScroll(eventP->data.sclRepeat.newValue-eventP->data.sclRepeat.value);
        return true;
    case keyDownEvent:
        vl=FldGetVisibleLines(getText())-1;
	switch (eventP->data.keyDown.chr) {
	case vchrPageUp:
        case 0x1700:
            updateScroll(-vl);
	    return true;
	case vchrPageDown:
        case 0x1701:
            updateScroll(vl);
	    return true;
	}
    default: break;
    }
    return false;
}


UInt16 Details::process() {
    loadData();
    FrmSetEventHandler(frmP,detailsEventHandler);
    Int16 r=FrmDoDialog(frmP);
    close();
    return r;
}
