/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "../ids.h"
#include "../APalmLib/Dialog.h"
#include "../Engine/Filter.h"
#include "../Engine/Locale.h"
#include "../UI/Channels.h"
#include "../UI/MainTable.h"

Char *DDLchan::getValue() {
    if (vCore->vFilter->currentGroup == (UInt16)-1) {
        return vCore->vLocale->sResMPAll;
    } else {
	return vCore->vFilter->groupNames.get(vCore->vFilter->currentGroup);
    }
}

void DDLchan::initList(ListPtr list) {
    ch.safeClear();
    ch.add(vCore->vLocale->sResMPAll);
    ch.addVector(vCore->vFilter->groupNames);
    ch.add(vCore->vLocale->sResMPGroupNew);
    if (vCore->vFilter->currentGroup != (UInt16)-1) {
	ch.add(vCore->vLocale->sResMPGroupEdit);
    }
    LstSetListChoices(list,ch.getArray(),ch.size());
}

void DDLchan::onClose() {
}

void DDLchan::onSelect(Int16 res) {
    if (res==-1) return;

    DlgChannels *di;

    if (res == (Int16)vCore->vFilter->groupNames.size()+1) {
        di=new DlgChannels(vCore, -1);
	di->process();
        vCore->vFilter->currentGroup = di->vGroupId;
        delete di;
    } else if (res == (Int16)vCore->vFilter->groupNames.size()+2) {
        di=new DlgChannels(vCore, vCore->vFilter->currentGroup);
	di->process();
        vCore->vFilter->currentGroup = di->vGroupId;
        delete di;
    } else {
        vCore->vFilter->currentGroup = res-1;
    }
    vCore->vMainTable->refreshData();
}

DDLchan::DDLchan(FormPtr form, Core *pCore) {
    vCore = pCore;
    init(form,idMainChannelsName,idMainChannelsList);
}

Char *DDLdate::getValue() {
    static char space[20];
    DateType dt;
    Int16 dow;

    switch (vCore->vFilter->Mode) {
    case 0:
        return vCore->vLocale->sResMPNow;
    case 1:
        return vCore->vLocale->sResMPToday;
    case 2:
        return vCore->vLocale->sResMPAll;
    case 3:
        DateDaysToDate(vCore->vFilter->Day,&dt);
        dow=(DayOfWeek(dt.month,dt.day,dt.year+1904)+6)%7;
        StrPrintF(space,"%02i.%02i, %s",dt.day,dt.month,vCore->vLocale->dayname(dow));
        return space;
    }
    return NULL;
}

void DDLdate::initList(ListPtr list) {
    Int16 res;
    Char *space;
    DateType dt;
    Int16 dow;

    dtext.add(vCore->vLocale->sResMPNow);
    dtext.add(vCore->vLocale->sResMPToday);
    dtext.add(vCore->vLocale->sResMPAll);
    for (res=0; res<(Int16)vCore->vTVdata->dates.size(); res++) {
	DateDaysToDate(vCore->vTVdata->dates.get(res),&dt);
	dow=(DayOfWeek(dt.month,dt.day,dt.year+1904)+6)%7;
	space=(Char *)MemPtrNew(20);
	if (space==NULL) ErrDisplay("Error allocate memory");
	StrPrintF(space,"%02i.%02i, %s",dt.day,dt.month,vCore->vLocale->dayname(dow));
	dtext.add(space);
    }
    LstSetListChoices(list,dtext.getArray(),dtext.size());
}

void DDLdate::onClose() {
    for (UInt16 i=3; i<dtext.size(); i++)
	MemPtrFree(dtext.get(i));
    dtext.safeClear();
}

void DDLdate::onSelect(Int16 res) {
    UInt32 begTime,endTime;

    if (res==-1) return;
    switch (res) {
    case 0:
	vCore->vFilter->Mode=0;
	break;
    case 1:
	vCore->vFilter->Mode=1;
	break;
    case 2:
	vCore->vFilter->Mode=2;
	break;
    default:
	vCore->vFilter->Mode=3;
	vCore->vFilter->Day=vCore->vTVdata->dates.get(res-3);
	break;
    }
    begTime=TimGetSeconds();
    vCore->vMainTable->refreshData();
    endTime=TimGetSeconds();
}

DDLdate::DDLdate(FormPtr form, Core *pCore) {
    vCore = pCore;
    init(form,idMainDatesName,idMainDatesList);
}

Char *DDLsel::getValue() {
    switch (vCore->vFilter->Selected) {
    case 0:
	return vCore->vLocale->sResMPAll;
    case 1:
	return vCore->vLocale->sResMPSel;
    case 2:
	return vCore->vLocale->sResMPUns;
    }
    return NULL;
}

void DDLsel::onSelect(Int16 res) {
    if (res==-1) return;
    vCore->vFilter->Selected=res;
    vCore->vMainTable->refreshData();
}

DDLsel::DDLsel(FormPtr form, Core *pCore) {
    vCore = pCore;
    init(form,idMainSelName,idMainSelList);
}
