/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.4 $ 
 *     $Date: 2003/10/24 14:21:37 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "../ids.h"
#include "MainTable.h"
#include "FontSel.h"
#include "../Engine/TVdata.h"
#include "../Engine/Filter.h"
#include "../Engine/Locale.h"
#include "Details.h"
#include "../APalmLib/HiRes.h"

MainTable::MainTable(FormPtr form, Core *pCore) {
    vCore = pCore;

    listChan=new DDLchan(form, vCore);
    listDate=new DDLdate(form, vCore);
    listSel=new DDLsel(form, vCore);

    listChan->show();
    listDate->show();
    listSel->show();

    AFntSetFont(vCore->FONTbase);
    init(form,idMainTable,idMainScroll);
    changeSize(false);

    Coord len1=AFntCharsWidth("00.00",5)+3;
    Coord len3=rowHeight+2;
    Coord len2=pos.extent.x-len1-len3;
    addColumn(len1);
    addColumn(len2);
    addColumn(len3);

    Int16 idcon,idcoff;
    if (rowHeight>=28) {
	idcon=idbmpcbon28x1;
	idcoff=idbmpcboff28x1;
    } else if (rowHeight>=22) {
	idcon=idbmpcbon22x1;
	idcoff=idbmpcboff22x1;
    } else if (rowHeight>=14) {
	idcon=idbmpcbon14x1;
	idcoff=idbmpcboff14x1;
    } else {
	idcon=idbmpcbon11x1;
	idcoff=idbmpcboff11x1;
    }
    bon=new Bitmap(idcon);
    boff=new Bitmap(idcoff);
}

MainTable::~MainTable() {
    delete bon;
    delete boff;

    delete listChan;
    delete listDate;
    delete listSel;

}

inline Int16 MainTable::getRowCount() {
    return vCore->vFilter->getRows();
}

void MainTable::refreshData() {
    upperRow=0;
    vCore->vFilter->recalc();
    redraw(true);
}

void MainTable::drawCell(Int16 row,Int16 col,RectangleType *rect) {
    char data[10];
    int t;
    Int16 cwidth,oldx;

    UInt8 *prog;
    UInt16 idchan;

    Char *pn;
    UInt16 len,lena;

    AWinEraseRectangle(rect,0);

    prog=vCore->vFilter->rowProgram(row);
    idchan=vCore->vFilter->rowChannel(row);

    if (prog==NULL) {
        switch (col) { // channel name output
        case 1:
	    AFntSetFontBold(vCore->FONTbase);
            Char *channelName=vCore->vTVdata->channelNames.get(idchan);
            AWinDrawTruncChars(channelName,StrLen(channelName),rect);
	    AFntSetFont(vCore->FONTbase);
            break;
        }
    } else {
        switch(col) { // program name output
	case 0:
            t = vCore->vTVdata->Program_getBegTime(prog)/60%(24L*60);
            StrPrintF(data,"%2i:%02i",t/60,t%60);
            cwidth=AFntCharsWidth(data,StrLen(data));
            oldx=rect->topLeft.x;
            rect->topLeft.x+=rect->extent.x-cwidth-3;
            AWinDrawTruncChars(data,StrLen(data),rect);
            rect->topLeft.x=oldx;
            break;
        case 1:
            pn=Program_getName(prog,&len);
            Program_getAnn(prog,&lena);
            if (lena>0) WinSetUnderlineMode(grayUnderline);
            AWinDrawTruncChars(pn,len,rect);
            if (lena>0) WinSetUnderlineMode(noUnderline);
            break;
        case 2:
            if (vCore->vTVdata->program_isChecked(prog,idchan))
                    bon->draw(rect->topLeft.x,rect->topLeft.y);
                else
                    boff->draw(rect->topLeft.x,rect->topLeft.y);
            break;
        }
    }
}

void MainTable::selectCell(Int16 row,Int16 col) {
    UInt8 *prog;
    UInt16 chanid;

    prog=vCore->vFilter->rowProgram(row);
    chanid=vCore->vFilter->rowChannel(row);

    if (prog==NULL) return;

    if (col==2) { // checkbox
        vCore->vTVdata->program_setChecked(prog,chanid,!(vCore->vTVdata->program_isChecked(prog,chanid)));
        redrawCell(row,col);
	vCore->vAlarmEngine->setNextAlarm(false);
        return;
    } else {
	Details *det=new Details(vCore);
	det->program = prog;
        det->channel = chanid;
        det->process();
        delete det;
    }
}

void MainTable::changeSize(Boolean needRedraw) {
    Coord width, height;
    AWinGetDisplayExtent(&width,&height);

    RectangleType orect;
    UInt16 i;

    i = FrmGetObjectIndex(form, idMainScroll);
    AFrmGetObjectBounds(form, i, &orect);
    orect.extent.y = height - orect.topLeft.y;
    AFrmSetObjectBounds(form, i, &orect);
    AWinEraseRectangle(&orect,0);

    i = FrmGetObjectIndex(form, idMainTable);
    AFrmGetObjectBounds(form, i, &orect);
    orect.extent.y = height - orect.topLeft.y;
    AFrmSetObjectBounds(form, i, &orect);

    resize();
    if (needRedraw)
	redraw(true);
//    char t[80];
//    StrPrintF(t,"h = %i",pos.extent.y);
//    ErrDisplay(t);
}
