/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.2 $ 
 *     $Date: 2003/10/09 17:19:13 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 

#include "Prefs.h"
#include "../ids.h"
#include "../Engine/AlarmEngine.h"
#include "../Engine/TVdata.h"
#include "FontSel.h"
#include "MainTable.h"
#include "../Engine/Filter.h"

Prefs::Prefs(Core *pCore) {
    vCore = pCore;
    init(idFormPreferences);
}

void Prefs::load() {
    ControlPtr l;

    l=(ControlPtr)ObjectByID(idPrefNowpop);
    CtlSetLabel(l,vCore->vLocale->timeToStr(vCore->vFilter->NowForward));
    l=(ControlPtr)ObjectByID(idPrefBegtime);
    h=vCore->vFilter->DayBegin/60/60; m=(vCore->vFilter->DayBegin/60)%60;
    StrPrintF(ou,"%i:%02i",h,m);
    CtlSetLabel(l,ou);
    FrmSetControlValue(frmP,FrmGetObjectIndex(frmP,idPrefChan),vCore->vFilter->ViewChan);
    FrmSetControlValue(frmP,FrmGetObjectIndex(frmP,idPrefSort),vCore->vFilter->ViewSort);

    l=(ControlPtr)ObjectByID(idPrefTimepop);
    if (vCore->vTVdata->detectGMT) {
        StrPrintF(ou3,"Auto");
    } else {
        if (vCore->vTVdata->coreGMToff==0)
            StrPrintF(ou3,"GMT");
	else
	    StrPrintF(ou3,"GMT%+i",(Int16)(vCore->vTVdata->coreGMToff/60/60));
    }
    CtlSetLabel(l,ou3);
}

void Prefs::save(UInt16 exitControl) {
    if (exitControl!=idPrefBtnOK) return;
    ControlPtr l;
    Char *a;

    l=(ControlPtr)ObjectByID(idPrefNowpop);
    vCore->vFilter->NowForward=vCore->vLocale->strToTime((Char *)CtlGetLabel(l));
    vCore->vFilter->DayBegin=(h*60L+m)*60L;
    vCore->vFilter->ViewChan=FrmGetControlValue(frmP,FrmGetObjectIndex(frmP,idPrefChan));
    vCore->vFilter->ViewSort=FrmGetControlValue(frmP,FrmGetObjectIndex(frmP,idPrefSort));
    l=(ControlPtr)ObjectByID(idPrefTimepop);
    a=(Char *)CtlGetLabel(l);
    vCore->vTVdata->detectGMT=StrCompare(a,"Auto")==0;
    vCore->vTVdata->coreGMToff=StrAToI(a+3)*60L*60L;

    vCore->vMainTable->refreshData();
    vCore->vAlarmEngine->setNextAlarm(false);
}

Boolean Prefs::eventHandler(EventPtr eventP) {
    switch (eventP->eType) {
    case ctlEnterEvent:
        if (eventP->data.ctlEnter.controlID==idPrefBegtime) {
            if (SelectOneTime(&h,&m,"Begin day time")) {
                StrPrintF(ou,"%i:%02i",h,m);
                CtlSetLabel(eventP->data.ctlEnter.pControl,ou);
            }
            return true;
        }
    default: break;
    }
    return false;
}
