/** 
 *     Caption: Zaval Advanced TV Guide 
 *     $Revision: 1.4 $ 
 *     $Date: 2003/10/09 17:19:12 $ 
 * 
 *     @author:     Alex Buloichik 
 *     @version:    2.2 
 * 
 * Zaval Advanced TV Guide (also known as ATV) is a small and nifty 
 * program that allows TV schedule viewing with your PDA, so you will 
 * be always informed on what's on TV. With our product you can do TV 
 * programs tracking and much more! 
 * 
 * For more info on this product read Zaval Advanced TV Guide User's Guide 
 * (It comes within this package). 
 * The latest product version is always available from the product's homepage: 
 * http://www.zaval.org/products/atv/ 
 * and from the SourceForge: 
 * http://sourceforge.net/projects/atv/ 
 * 
 * Contacts: 
 *   Support : support@zaval.org 
 *   Change Requests : change-request@zaval.org 
 *   Feedback : feedback@zaval.org 
 *   Other : info@zaval.org 
 * 
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * (version 2) as published by the Free Software Foundation. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 */ 
#include "ids.h"
#include "UI/MainTable.h"
#include "APalmLib/Utils.h"
#include "Engine/Core.h"
#include "UI/Prefs.h"
#include "UI/FontSel.h"
#include "UI/AlarmDialog.h"

static void MenuMainDoCommand(UInt16 command) {
    FormPtr frmP;
    Dialog *di;
    FontSel *dif;

    switch (command) {
    case idMenuItPreferences:
        di=new Prefs(core);
        di->process();
        delete di;
	break;
    case idMenuItAlarms:
        di=new AlarmDialog(core->vAlarmEngine);
        di->process();
        delete di;
	break;
    case idMenuItFonts:
        dif=new FontSel(core);
	dif->process();
	if (dif->changed) {
            core->FONTbase = dif->newFont;
	    core->reCreateMainTable();
	}
        delete dif;
        break;
    case idMenuItDelete:
        if (FrmAlert(idAlertDelete)!=0) break;
        core->vTVdata->del();
        core->vFilter->del();
        core->vMainTable->refreshData();
	core->vAlarmEngine->setNextAlarm(false);
	break;
    case idMenuItAbout:
	frmP = FrmInitForm(idFormAbout);
	FrmDoDialog(frmP);
	FrmDeleteForm(frmP);
	break;
    default:
        ErrDisplay("Error menu item selected");
        break;
    }
}

static Boolean MainFormHandlerEvent(EventPtr eventP) { // plain handler
    Boolean handled=false;

    switch (eventP->eType) {

    case frmGadgetEnterEvent:
    case keyDownEvent:
    case sclRepeatEvent:
    case sclEnterEvent:
    case sclExitEvent:
        return core->vMainTable->event(eventP);

    case menuEvent:
        MenuMainDoCommand(eventP->data.menu.itemID);
	handled=true;
	break;
    case frmOpenEvent:
        FrmDrawForm(FrmGetActiveForm());
	core->setScreenBound();
        core->vMainTable->refreshData();
	handled=true;
	break;
    case fldEnterEvent:
        switch (eventP->data.fldEnter.fieldID) {
        case idMainChannelsName:
            core->vMainTable->listChan->process();
            handled=true;
            break;
        case idMainDatesName:
            core->vMainTable->listDate->process();
            handled=true;
            break;
        case idMainSelName:
            core->vMainTable->listSel->process();
            handled=true;
            break;
        }
    default:
        handled = core->vMainTable->event(eventP);
        break;
    }
    return handled;
}

static Boolean AppHandleEvent(EventPtr eventP) {
    UInt16 formId;
    FormPtr frmP;
    
    if (eventP->eType == frmLoadEvent) {
        formId = eventP->data.frmLoad.formID;
        frmP = FrmInitForm(formId);
        FrmSetActiveForm(frmP);
        switch (formId) {
	case idFormMain:
	    core->vMainTable = new MainTable(frmP, core);
            FrmSetEventHandler(frmP,MainFormHandlerEvent);
            break;
        default:
            ErrNonFatalDisplay("Invalid Form Load Event");
            break;
        }
        return true;
    }
    return false;
}

static void EventLoop() {
    UInt16 error;
    EventType event;

    do {
	EvtGetEvent(&event, evtWaitForever);
        if (!SysHandleEvent(&event))
            if (!MenuHandleEvent(NULL,&event,&error))
                if (!AppHandleEvent(&event))
                    FrmDispatchEvent(&event);
    } while (event.eType != appStopEvent);
}

UInt32 PilotMain (UInt16 cmd, void *cmdPBP, UInt16 launchFlags) {
    Core *tempCore;
    Details *alarmDetails;
    UInt32 alarmTime;

    switch (cmd) {
    case sysAppLaunchCmdNormalLaunch:
	new Core(false); // Core::Core() stores self into global 'core' variable
        FrmGotoForm(idFormMain);
	EventLoop();
	FrmSetEventHandler(core->vMainTable->getForm(),NULL);
        delete core;
	FrmCloseAllForms();
        break;
    case sysAppLaunchCmdAlarmTriggered:
        break;
    case sysAppLaunchCmdDisplayAlarm:
	alarmDetails = (Details *)(((SysDisplayAlarmParamType *)cmdPBP)->ref);
        alarmTime = ((SysDisplayAlarmParamType *)cmdPBP)->alarmSeconds;
	if (!(launchFlags & sysAppLaunchFlagSubCall)) {
	    tempCore = new Core(true);
	    tempCore->vAlarmEngine->playAlarm();
	    tempCore->vAlarmEngine->display(alarmTime);
	    tempCore->vAlarmEngine->setNextAlarm(false);
            delete tempCore;
	} else {
	    core->vAlarmEngine->playAlarm();
	    core->vAlarmEngine->display(alarmTime);
	    core->vAlarmEngine->setNextAlarm(false);
	}
        break;
/*    case sysAppLaunchCmdFind:
        findRun((FindParamsPtr)cmdPBP);
        break;*/
    }
    return 0;
}

// HostFPrintF(HostLogFile(),"test");
